==> adms-2.2.4.tar.gz: adms-2.2.4/admsFile/fileMain.c <==
/*
 * RCS Info
 * $Id: fileMain.c,v 1.20 2005/11/01 14:11:14 r29173 Exp $
 * 
 * Log
 * $Log: fileMain.c,v $
 * Revision 1.20  2005/11/01 14:11:14  r29173
 * fixed spelling
 *
 * Revision 1.19  2005/06/17 16:44:27  r29173
 * debug memory allocation with valgrind
 *
 * Revision 1.18  2005/05/09 14:38:30  r29173
 * cleaned-up source directory structure
 *
 * Revision 1.17  2005/05/03 09:35:15  r29173
 * cleaned-up header file dependencies
 *
 * Revision 1.16  2005/05/03 07:58:20  r29173
 * moved admsNS into admsFile
 *
 * Revision 1.15  2005/04/21 07:54:49  r29173
 * removed some usages of adms_strconcat
 *
 * Revision 1.14  2005/03/23 13:38:33  r29173
 * migration to adms-2.x.x: added xpath parser
 *
 * Revision 1.13  2005/01/11 09:58:00  r29173
 * tons of changes - simplify accessor names (valueof, valueto)
 *
 * Revision 1.12  2004/11/19 12:27:50  r29173
 * improved parsing of command line
 *
 * Revision 1.11  2004/11/10 13:32:22  r29173
 * did minor clean-up
 *
 * Revision 1.10  2004/11/01 09:37:37  r29173
 * changed messaging macros (avoid useless computation of arguments)
 * improved the coding of transforms in xmlProgram.c
 *
 * Revision 1.9  2004/10/27 16:11:39  r29173
 * release 1.37.0
 * tons of changes - modify the way settors and gettors are built
 * from adms.xml in order to simplify admsProgram.c
 * added zspice to testbench
 * spectre/ads/mica/zspice/simkit passed
 *
 * Revision 1.8  2004/10/20 15:33:51  r29173
 * tons of changes
 * 1- introduced element whileloop (marat)
 * 2- reshaped the handling of the messaging system
 * 3- fixed bug in admsCheck/Makefile.am
 * 4- started implementation of implicit adms transforms
 *
 * Revision 1.7  2004/10/13 15:24:13  r29173
 * preprocess xml files passed to admsXml with adms_file_getpathname
 *
 * Revision 1.6  2004/09/23 16:20:48  r29173
 * create automatically standard vams files when missing.
 *
 * Revision 1.5  2004/09/07 07:38:46  r29173
 * cleaned-up the messaging system
 *
 * Revision 1.4  2004/08/19 16:43:13  r29173
 * cleaned-up i/o file handling
 *
 * Revision 1.3  2004/08/16 10:52:29  r29173
 * changed handling of adms_message_verbose (created adms_message_debug)
 *
 * Revision 1.2  2004/08/14 17:01:02  marat_yakupov
 * Added default -I /install_dir/include to admsXml
 *
 * Revision 1.1  2004/08/03 12:33:54  r29173
 * import adms-1.21.0 from local CVS
 *
 * Revision 1.2  2004/05/26 13:02:48  r29173
 * added default values to all enumerations
 *
 * Revision 1.1.1.1  2004/05/21 12:20:01  r29173
 * recreated cvs data structure (crashed after revision 1.13.0!)
 *
 * Revision 1.7  2004/03/08 20:47:48  r29173
 * removed admsbegin/admsend constructs
 *
 * Revision 1.6  2004/03/08 13:58:05  r29173
 * all code lower-cased
 *
 * Revision 1.5  2004/02/19 17:02:38  r29173
 * started implementing code builder in yacc parser
 *
 * Revision 1.4  2004/02/13 14:28:40  r29173
 * started implementing code builder in yacc parser
 *
 * Revision 1.3  2004/02/10 10:48:25  r29173
 * started implementing code builder in yacc parser
 *
 * Revision 1.2  2004/01/13 11:26:15  r29173
 * added adms_message_usage
 *
 * Revision 1.1  2004/01/09 16:26:45  r29173
 * cleaned-up use of #include
 *
 * Revision 1.10  2004/01/06 12:35:10  r29173
 * fixed the use of globals: input file and output file
 *
 * Revision 1.9  2004/01/05 11:40:29  r29173
 * removed unused code
 *
 * Revision 1.8  2003/12/12 15:08:34  r29173
 * changed construct (a==NULL) to (!a)
 *
 * Revision 1.7  2003/12/12 14:43:37  r29173
 * changed construct (a==NULL) to (!a)
 *
 * Revision 1.6  2003/12/12 14:33:26  r29173
 * changed construct (a!=NULL) into (a) or (a)?1:0
 *
 * Revision 1.5  2003/12/11 20:14:08  r29173
 * cleaned-up after compilation with CFLAG+=-Wall
 *
 * Revision 1.4  2003/12/11 16:01:53  r29173
 * changed prefix [epst]_adms_ to [epst]_
 *
 * Revision 1.3  2003/11/06 12:45:08  r29173
 * tried to improve/fixed setting of nodes/variables/expressions units
 * NOTE: v-1.1.16 of SPevaluate.include differs from v-1.1.15
 *
 * Revision 1.2  2003/05/21 14:18:01  r29173
 * add rcs info
 *
 */

#include <admsFile/admsFile.h>
#include <admsObject/admsC.h>

FILE *adms_file_output;

typedef struct s_filehandling_ipath t_filehandling_ipath;
typedef t_filehandling_ipath*       p_filehandling_ipath;
struct s_filehandling_ipath {
  char *filename;
  FILE * fh;
};
static char* adms_strconcat (const char *string1, ...)
{
  gsize l;     
  va_list args;
  char *s;
  char *concat;
  char *ptr;
  if (!string1)
    G_BREAKPOINT();
  l = 1 + strlen (string1);
  va_start (args, string1);
  s = va_arg (args, gchar*);
  while (s)
  {
    l += strlen (s);
    s = va_arg (args, gchar*);
  }
  va_end (args);
  concat = g_new (gchar, l);
  ptr = concat;
  ptr = g_stpcpy (ptr, string1);
  va_start (args, string1);
  s = va_arg (args, gchar*);
  while (s)
  {
    ptr = g_stpcpy (ptr, s);
    s = va_arg (args, gchar*);
  }
  va_end (args);
  return concat;
}
gboolean adms_file_print_usage (const p_file_usage myusage)
{
  myusage ();
  adms_message_usage ("switches: type %s -h for more info\n", g_get_prgname())
  return 0;
}
gboolean adms_file_check_command_line (const int argc, const char **argv, const p_file_usage myusage)
{
  if(
    ((argc == 2) && !strcmp (argv[1],"-v"))
    ||
    ((argc == 2) && !strcmp (argv[1],"--version"))
    ||
    ((argc == 3) && !strcmp (argv[1],"-") && !strcmp (argv[2],"v"))
    ||
    ((argc == 3) && !strcmp (argv[1],"--") && !strcmp (argv[2],"version"))
  )
    {
      adms_message_usage ("<release name=\"%s\" version=\"%s\" date=\"%s\" time=\"%s\"/>\n",
        g_get_prgname(), PACKAGE_VERSION, __DATE__, __TIME__)
      return 1;
    }
  else if(
    ((argc == 2) && !strcmp (argv[1],"-i"))
    ||
    ((argc == 2) && !strcmp (argv[1],"--info"))
    ||
    ((argc == 3) && !strcmp (argv[1],"-") && !strcmp (argv[2],"i"))
    ||
    ((argc == 3) && !strcmp (argv[1],"--") && !strcmp (argv[2],"info"))
  )
    {
      adms_message_usage ("<info\n",PACKAGE_BUGREPORT)
      adms_message_usage ("  author=\"laurent lemaitre\"\n",PACKAGE_BUGREPORT)
      adms_message_usage ("  bug-report=\"%s\"\n",PACKAGE_BUGREPORT)
      adms_message_usage ("  home-page=\"http://sourceforge.net/projects/mot-adms\"\n")
      adms_message_usage ("  mailing-list=\"mot-adms-users@lists.sourceforge.net\">\n")
      adms_message_usage ("  <release name=\"%s\" version=\"%s\" date=\"%s\" time=\"%s\"/>\n",
        g_get_prgname(), PACKAGE_VERSION, __DATE__, __TIME__);
      adms_message_usage ("  <dependency>\n")
      adms_message_usage ("    <library name=\"glib-2\" version=\"%i.%i.%i\">\n",
        GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
      adms_message_usage ("      <source location=\"http://www.gtk.org/\"/>\n")
      adms_message_usage ("    </library>\n")
      adms_message_usage ("  </dependency>\n")
      adms_message_usage ("</info>\n")
      return 1;
    }
  else if(
    ((argc == 2) && !strcmp (argv[1],"-h"))
    ||
    ((argc == 2) && !strcmp (argv[1],"--help"))
    ||
    ((argc == 3) && !strcmp (argv[1],"-") && !strcmp (argv[2],"h"))
    ||
    ((argc == 3) && !strcmp (argv[1],"--") && !strcmp (argv[2],"help"))
  )
    {
      myusage ();
      adms_message_usage ("-i or -info: info on the tools (release, web site, mailing list)\n")
      adms_message_usage ("-v or -version: version number\n")
      adms_message_usage ("-h or -help: short help\n")
      return 1;
    }
  else
    return 0;
}
void adms_file_setfile_output (FILE *ofh)
{
  adms_file_output = ofh;
}
FILE *adms_file_getfile_output (void)
{
  return adms_file_output;
}
void adms_file_close_output(void)
{
  adms_message_verbose ("closing output file\n")
  fclose(adms_file_getfile_output());
}
gboolean adms_file_filehandling_print(const char *str)
{
  FILE *ofh;
  ofh = adms_file_getfile_output();
  g_return_val_if_fail(ofh,TRUE);
  fputs ("",ofh);
  fputs (str,ofh);
  fflush (ofh);
  return FALSE;
}
static void adms_path_split_cb (p_path mypath,char *mychars,char *mynext)
{
  int mysize=mychars-mynext;
  char*mytoken=g_strndup(mynext,mysize);
  adms_message_debug("[%-15s] path->_token:push: %s (%i chars)\n",__FUNCTION__,mytoken,mysize)
  adms_path_list_token_prepend_by_id(mypath,mytoken); 
}
static void adms_path_split (p_path mypath)
{
  char*mychars=adms_path_valueof_value(mypath);
  char*mynext=mychars;
  while(*mychars!='\0')
  {
    adms_message_debug("[%-15s] %c (%i)\n",__FUNCTION__,*mychars,mychars-mynext)
    if((*mychars=='/')||(*mychars=='\\'))
    {
      adms_path_split_cb (mypath,mychars,mynext);
      mynext=mychars+1;
    }
    mychars++;
  }
  adms_path_split_cb (mypath,mychars,mynext);
  adms_path_valueof_token(mypath)=g_slist_reverse(adms_path_valueof_token(mypath));
  adms_message_debug("[%-15s] path->_token: reverse\n",__FUNCTION__)
}
static char *adms_path_join (p_path mypath, const char *sep,int start)
{
  char*str=NULL;
  GSList*list=g_slist_nth(adms_path_valueof_token(mypath),start);
  while(list)
  {
    str=str?adms_strconcat(str,list->data,NULL):adms_strconcat(list->data,NULL);
    list=list->next;
    if(list)
      str=adms_strconcat(str,sep,NULL);
    adms_message_debug("[%-15s] new path %s\n",__FUNCTION__,str)
  }
  if(!str)
    str=g_strdup("");
  return str;
}
static char *adms_path_towindows_cb (p_path mypath)
{
  char *cygroot;
  if(getenv("cygroot"))
  {
    adms_message_verbose("try cygroot path '%s'\n",cygroot)
    cygroot=adms_strconcat(cygroot,"\\",adms_path_join(mypath,"\\",0),NULL);
    return cygroot;
  }
  else
  {
    char *cygdrive="cbdefghijklmnopqrstuvwxyz";
    while(*cygdrive!='\0')
    {
      cygroot=g_strdup_printf("%c:\\cygwin",*cygdrive);
      adms_message_verbose("try cygroot path '%s'\n",cygroot)
      if (g_file_test (cygroot, G_FILE_TEST_IS_DIR))
        return cygroot;
      else
        cygdrive++;
    }
    return NULL;
  }
}
static char *adms_path_towindows (p_path mypath)
{
  if(((char*)adms_path_valueof_token(mypath)->data)[0]=='\0')
  {
    char *cygroot;
    adms_message_verbose("[%-15s] cygpath found\n",__FUNCTION__)
    if ((cygroot=adms_path_towindows_cb (mypath)))
      return adms_strconcat(cygroot,"\\",adms_path_join(mypath,"\\",0),NULL);
    else
    {
      adms_message_error("environment variable `cygroot' not set\n")
      adms_message_error("Please set `cygroot' to the path of your cygwin installation\n")
      adms_message_fatal("Example: export cygroot='c:\\cygwin' (please keep the ')\n")
    }
  }
  else if((g_slist_length(adms_path_valueof_token(mypath))>2)
    &&(!strcmp(g_slist_nth_data(adms_path_valueof_token(mypath),1),"cygdrive")))
  {
    adms_message_verbose("[%-15s]: cygwin path found\n",__FUNCTION__)
    return adms_strconcat(g_slist_nth_data(adms_path_valueof_token(mypath),2),":\\",adms_path_join(mypath,"\\",3),NULL);
  }
  else
  {
    adms_message_verbose("[%-15s]: windows path found\n",__FUNCTION__)
    return adms_path_join(mypath,"\\",0);
  }
}
char *adms_file_getpathname(const char *name)
{
  char*mypath;
  p_path myPath=adms_path_new(g_strdup(name));
  adms_message_verbose ("processing pathname '%s'\n",name)
  adms_path_split(myPath);
#if defined(WIN32)
  adms_message_verbose (" (os windows)\n")
  mypath=adms_path_towindows(myPath);
#else
  adms_message_verbose (" (os linux)\n")
  mypath=adms_path_join(myPath,"/",0);
#endif
  adms_path_free(myPath);
  return mypath;
}
static const char *adms_file_option_getstring_type (int mytype)
{
  if (mytype==G_FILE_TEST_IS_REGULAR)
    return "regular file";
  else if (mytype==G_FILE_TEST_IS_DIR)
    return "directory";
  else
    adms_message_fatal ("[internal-%s]: should not be reached\n",__FUNCTION__)
}
GSList *adms_file_getlist_from_argv (const int argc,const char **argv,const char opt,int optType,GSList*myList)
{
  int i;
  for (i=1; i<argc; i++)
  {
    char*shortOpt=g_strdup_printf("-%c",opt);
    char*newValue=NULL;
    char*value=g_strdup (argv[i]);
    if (!strcmp("-f",shortOpt))
    {
      if ((strlen(value)>2)&&((value[0])=='-'))
        {}
      else if ((strlen(value)==2)&&((value[0])=='-'))
        i++;
      else
        newValue=g_strdup(argv[i]);
    }
    if (!strcmp(value,shortOpt))
    {
      if (i<argc-1)
        newValue = g_strdup (argv[i+1]);
      else
        adms_message_fatal ("[cmdline option %s]: option has no value - expecting %s\n",
          shortOpt,adms_file_option_getstring_type(optType))
    }
    else if ((strlen(value)>2)&&((value[0])=='-')&&(value[1]==opt))
      newValue = g_strndup (value+2,strlen(value)-2);
    if (newValue)
    {
      char *newValueOS=adms_file_getpathname(newValue);
      if (g_file_test (newValueOS,optType))
        myList = g_slist_prepend(myList,newValueOS);
      else
        adms_message_fatal ("[cmdline option %s]: value not valid '%s' - expecting %s\n",
          shortOpt,newValue,adms_file_option_getstring_type(optType))
      adms_message_verbose ("[cmdline option %s]: append '%s' [%s]\n",
          shortOpt,newValueOS,adms_file_option_getstring_type(optType))
    }
    free(shortOpt);
    free(value);
  }
  return g_slist_reverse(myList);
}
GSList *adms_file_get_ifile_from_argv (const int argc,const char **argv)
{
  return adms_file_getlist_from_argv (argc,argv,'f',G_FILE_TEST_IS_REGULAR,NULL);
}
GSList *adms_file_get_ipath_from_argv (const int argc,const char **argv)
{
  return adms_file_getlist_from_argv (argc,argv,'I',G_FILE_TEST_IS_DIR,g_slist_prepend(NULL,"."));
}
GSList *adms_file_get_program_from_argv (const int argc,const char **argv)
{
  return adms_file_getlist_from_argv (argc,argv,'e',G_FILE_TEST_IS_REGULAR,NULL);
}
static FILE * adms_file_open (const char *fileName,const char *permissions)
{
  FILE *fh;
  char *admsfilename=adms_file_getpathname(fileName);
  adms_message_verbose ("trying to open file ... '%s' [mode %s]\n",admsfilename,permissions)
  fh = fopen (admsfilename,permissions);
  free(admsfilename);
  return fh;
}
FILE * adms_file_open_read (const char *fileName, const char *message)
{
  FILE *fh=adms_file_open (fileName,"r");
  if (!fh)
    adms_message_fatal ("[%s]: failed to open file for reading ... '%s'\n",message,fileName)
  return fh;
}
FILE * adms_file_open_write (const char *fileName, const char *message)
{
  FILE *fh=adms_file_open (fileName,"wb");
  if (!fh)
    adms_message_fatal ("[%s]: failed to open file for writing ... '%s'\n",message,fileName)
  return fh;
}
static void adms_file_open_read_with_path_cb ( char *pathName, const p_filehandling_ipath ipath)
{
  char *buildname=g_build_filename(pathName,ipath->filename,NULL);
  if (!(ipath->fh))
    ipath->fh = adms_file_open (buildname,"r");
}
FILE * adms_file_open_read_with_path (const char *fileName, GSList * path)
{
  p_filehandling_ipath ipath;
  ipath = g_new0(t_filehandling_ipath,1);
  ipath->filename = g_strdup(fileName);
  g_slist_foreach ( path, (GFunc) adms_file_open_read_with_path_cb, ipath);
  return ipath->fh;
}
char *adms_path_get_basename (const char *name)
{
  char *basename;
  p_path mypath=adms_path_new(g_strdup(name));
  adms_path_split(mypath);
  basename=g_slist_last(adms_path_valueof_token(mypath))->data;
  free(mypath);
  return basename;
}

==> adms-2.2.5.tar.gz: adms-2.2.5/admsFile/fileMain.c <==
/*
 * RCS Info
 * $Id: fileMain.c 537 2007-03-01 14:10:34Z r29173 $
 * 
 * Log
 * $Log$
 * Revision 1.25  2006/09/13 16:13:20  r29173
 * started migration to www.antlr.org tools
 *
 * Revision 1.24  2006/08/10 11:31:47  r29173
 * did lot of simplifications
 *
 * Revision 1.23  2006/08/04 17:15:31  r29173
 * rearranged code partionning into files
 *
 * Revision 1.22  2006/08/02 16:45:53  r29173
 * rearranged code partionning into files
 *
 * Revision 1.21  2006/06/16 12:24:25  r29173
 * ton of changes: memory clean-up
 *
 * Revision 1.20  2005/11/01 14:11:14  r29173
 * fixed spelling
 *
 * Revision 1.19  2005/06/17 16:44:27  r29173
 * debug memory allocation with valgrind
 *
 * Revision 1.18  2005/05/09 14:38:30  r29173
 * cleaned-up source directory structure
 *
 * Revision 1.17  2005/05/03 09:35:15  r29173
 * cleaned-up header file dependencies
 *
 * Revision 1.16  2005/05/03 07:58:20  r29173
 * moved admsNS into admsFile
 *
 * Revision 1.15  2005/04/21 07:54:49  r29173
 * removed some usages of adms_strconcat
 *
 * Revision 1.14  2005/03/23 13:38:33  r29173
 * migration to adms-2.x.x: added xpath parser
 *
 * Revision 1.13  2005/01/11 09:58:00  r29173
 * tons of changes - simplify accessor names (valueof, valueto)
 *
 * Revision 1.12  2004/11/19 12:27:50  r29173
 * improved parsing of command line
 *
 * Revision 1.11  2004/11/10 13:32:22  r29173
 * did minor clean-up
 *
 * Revision 1.10  2004/11/01 09:37:37  r29173
 * changed messaging macros (avoid useless computation of arguments)
 * improved the coding of transforms in xmlProgram.c
 *
 * Revision 1.9  2004/10/27 16:11:39  r29173
 * release 1.37.0
 * tons of changes - modify the way settors and gettors are built
 * from adms.xml in order to simplify admsProgram.c
 * added zspice to testbench
 * spectre/ads/mica/zspice/simkit passed
 *
 * Revision 1.8  2004/10/20 15:33:51  r29173
 * tons of changes
 * 1- introduced element whileloop (marat)
 * 2- reshaped the handling of the messaging system
 * 3- fixed bug in admsCheck/Makefile.am
 * 4- started implementation of implicit adms transforms
 *
 * Revision 1.7  2004/10/13 15:24:13  r29173
 * preprocess xml files passed to admsXml with adms_file_unixify
 *
 * Revision 1.6  2004/09/23 16:20:48  r29173
 * create automatically standard vams files when missing.
 *
 * Revision 1.5  2004/09/07 07:38:46  r29173
 * cleaned-up the messaging system
 *
 * Revision 1.4  2004/08/19 16:43:13  r29173
 * cleaned-up i/o file handling
 *
 * Revision 1.3  2004/08/16 10:52:29  r29173
 * changed handling of adms_message_verbose (created adms_message_debug)
 *
 * Revision 1.2  2004/08/14 17:01:02  marat_yakupov
 * Added default -I /install_dir/include to admsXml
 *
 * Revision 1.1  2004/08/03 12:33:54  r29173
 * import adms-1.21.0 from local CVS
 *
 * Revision 1.2  2004/05/26 13:02:48  r29173
 * added default values to all enumerations
 *
 * Revision 1.1.1.1  2004/05/21 12:20:01  r29173
 * recreated cvs data structure (crashed after revision 1.13.0!)
 *
 * Revision 1.7  2004/03/08 20:47:48  r29173
 * removed admsbegin/admsend constructs
 *
 * Revision 1.6  2004/03/08 13:58:05  r29173
 * all code lower-cased
 *
 * Revision 1.5  2004/02/19 17:02:38  r29173
 * started implementing code builder in yacc parser
 *
 * Revision 1.4  2004/02/13 14:28:40  r29173
 * started implementing code builder in yacc parser
 *
 * Revision 1.3  2004/02/10 10:48:25  r29173
 * started implementing code builder in yacc parser
 *
 * Revision 1.2  2004/01/13 11:26:15  r29173
 * added adms_message_usage
 *
 * Revision 1.1  2004/01/09 16:26:45  r29173
 * cleaned-up use of #include
 *
 * Revision 1.10  2004/01/06 12:35:10  r29173
 * fixed the use of globals: input file and output file
 *
 * Revision 1.9  2004/01/05 11:40:29  r29173
 * removed unused code
 *
 * Revision 1.8  2003/12/12 15:08:34  r29173
 * changed construct (a==NULL) to (!a)
 *
 * Revision 1.7  2003/12/12 14:43:37  r29173
 * changed construct (a==NULL) to (!a)
 *
 * Revision 1.6  2003/12/12 14:33:26  r29173
 * changed construct (a!=NULL) into (a) or (a)?1:0
 *
 * Revision 1.5  2003/12/11 20:14:08  r29173
 * cleaned-up after compilation with CFLAG+=-Wall
 *
 * Revision 1.4  2003/12/11 16:01:53  r29173
 * changed prefix [epst]_adms_ to [epst]_
 *
 * Revision 1.3  2003/11/06 12:45:08  r29173
 * tried to improve/fixed setting of nodes/variables/expressions units
 * NOTE: v-1.1.16 of SPevaluate.include differs from v-1.1.15
 *
 * Revision 1.2  2003/05/21 14:18:01  r29173
 * add rcs info
 *
 */

#include <admsFile/admsFile.h>
#include <admsObject/admsC.h>

FILE* adms_file_output;
void adms_file_setfile_output (FILE* ofh)
{
  adms_file_output=ofh;
}
void adms_file_close_output(void)
{
  adms_message_verbose(("closing output file\n"))
  fclose(adms_file_output);
}
int adms_file_filehandling_print(const char* str)
{
  fputs(str,adms_file_output);
  fflush(adms_file_output);
  return 0;
}

static inline FILE* adms_file_open (const char *myfilename,const char *mypermissions)
{
  adms_message_verbose(("open file '%s' [mode %s]\n",myfilename,mypermissions))
  return fopen(myfilename,mypermissions);
}
FILE* adms_file_open_read (const char *myfilename)
{
  FILE* fh=adms_file_open(myfilename,"r");
  if(!fh)
    adms_message_fatal(("%s: failed to open file [reading mode]\n",myfilename))
  return fh;
}
FILE* adms_file_open_read_with_path (const char *myfilename, p_slist mypath)
{
  FILE* myfh=NULL;
  p_slist l; for(l=mypath;l;l=l->next)
  {
    char* mypathname=strdup((char*)(l->data));
    adms_strconcat2(&mypathname,ADMS_PATH_SEPARATOR);
    adms_strconcat2(&mypathname,myfilename);
    if((myfh=adms_file_open(mypathname,"r")))
      return myfh;
  }
  return NULL;
}
FILE* adms_file_open_write (const char *myfilename)
{
  FILE* fh=adms_file_open(myfilename,"wb");
  if(!fh)
    adms_message_fatal(("failed to open file [write mode]\n",myfilename))
  return fh;
}
